<?php
namespace IGFW\Models\Gateways;

use IGFW\Helpers\Plugin_Constants;

if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

/**
 * Model that houses the Invoice Payment Gateway.
 *
 * @since 1.0.0
 */
class IGFW_Invoice_Gateway extends \WC_Payment_Gateway {

    /**
     * Constructor for the gateway.
     *
     * @since 1.0.0
     * @access public
     */
    public function __construct() {

        $this->id = "igfw_invoice_gateway";
        $this->icon = apply_filters('igfw_invoice_gateway_icon', '');
        $this->method_title = __('Invoice Gateway', 'invoice-gateway-for-woocommerce');
        $this->method_description = __('Have your customers pay with and invoice generated by your accounting system. You will be responsible for invoicing the customer and entering the invoice ID on their order. Then once you confirm payment is received, you can mark the order as Completed as normal to indicate that it has been paid.', 'invoice-gateway-for-woocommerce');

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        // Get settings
        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->instructions = $this->get_option('instructions', $this->description);
        $this->enable_for_methods = $this->get_option('enable_for_methods', array());
        $this->enable_for_virtual = $this->get_option('enable_for_virtual', 'yes') === 'yes' ? true : false;

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_thankyou_igfw_invoice_gateway', array($this, 'thankyou_page'));

        // Customer Emails
        add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);

        do_action('igfw_invoice_gateway_construct');

    }

    /**
     * Initialise Gateway Settings Form Fields.
     *
     * @since 1.0.0
     * @access public
     */
    public function init_form_fields() {

        $shipping_methods = array();

        if (is_admin()) {

            foreach (WC()->shipping()->load_shipping_methods() as $method) {

                if (method_exists($method, 'get_method_title')) {
                    $shipping_methods[$method->id] = $method->get_method_title();
                } elseif (property_exists($method, 'method_title')) {
                    $shipping_methods[$method->id] = $method->method_title;
                } elseif (property_exists($method, 'title')) {
                    $shipping_methods[$method->id] = $method->title;
                }

            }

        }

        $this->form_fields = apply_filters('igfw_invoice_gateway_form_fields', array(
            'enabled' => array(
                'title' => __('Enable Invoice Gateway', 'invoice-gateway-for-woocommerce'),
                'label' => __('Enable Invoice Payment Gateway', 'invoice-gateway-for-woocommerce'),
                'type' => 'checkbox',
                'description' => '',
                'default' => 'no',
            ),
            'title' => array(
                'title' => __('Title', 'invoice-gateway-for-woocommerce'),
                'type' => 'text',
                'description' => __('Payment method description that the customer will see on your checkout.', 'invoice-gateway-for-woocommerce'),
                'default' => __('Invoice Gateway', 'invoice-gateway-for-woocommerce'),
                'desc_tip' => true,
            ),
            'description' => array(
                'title' => __('Description', 'invoice-gateway-for-woocommerce'),
                'type' => 'textarea',
                'description' => __('Payment method description that the customer will see on your website.', 'invoice-gateway-for-woocommerce'),
                'default' => __('Pay with invoice.', 'invoice-gateway-for-woocommerce'),
                'desc_tip' => true,
            ),
            'instructions' => array(
                'title' => __('Instructions', 'invoice-gateway-for-woocommerce'),
                'type' => 'textarea',
                'description' => __('Instructions that will be added to the thank you page.', 'invoice-gateway-for-woocommerce'),
                'default' => __('Pay with invoice.', 'invoice-gateway-for-woocommerce'),
                'desc_tip' => true,
            ),
            'enable_for_methods' => array(
                'title' => __('Enable for shipping methods', 'invoice-gateway-for-woocommerce'),
                'type' => 'multiselect',
                'class' => 'wc-enhanced-select',
                'css' => 'width: 450px;',
                'default' => '',
                'description' => __('If invoice gateway is only available for certain methods, set it up here. Leave blank to enable for all methods.', 'invoice-gateway-for-woocommerce'),
                'options' => $shipping_methods,
                'desc_tip' => true,
                'custom_attributes' => array(
                    'data-placeholder' => __('Select shipping methods', 'invoice-gateway-for-woocommerce'),
                ),
            ),
            'enable_for_virtual' => array(
                'title' => __('Accept for virtual orders', 'invoice-gateway-for-woocommerce'),
                'label' => __('Accept invoice if the order is virtual', 'invoice-gateway-for-woocommerce'),
                'type' => 'checkbox',
                'default' => 'yes',
            ),
        ));

    }

    /**
     * Show Purchase Order Number option
     *
     * @since 1.1
     */
    public function payment_fields() {

        if (get_option('igfw_enable_purchase_order_number') == 'yes') {

            echo '<p><b>' . __('Purchase Order (optional)', 'invoice-gateway-for-woocommerce') . '</b></p>';
            echo '<p><input type="text" name="igfw_purchase_order_number" placeholder="PO Number"></p>';
            echo '<p>' . __('We will generate and send you an invoice for your order, if you have a PO number, please enter it.', 'invoice-gateway-for-woocommerce') . '</p>';

        } else {
            $description = $this->get_description();
            if ($description) {
                echo wpautop(wptexturize(esc_attr($description))); // @codingStandardsIgnoreLine.
            }
        }

    }

    /**
     * Check If The Gateway Is Available For Use.
     *
     * @since 1.0.0
     * @access public
     * @return bool
     */
    public function is_available() {

        $order = null;
        $needs_shipping = false;

        // Test if shipping is needed first
        if (WC()->cart && WC()->cart->needs_shipping()) {
            $needs_shipping = true;
        } elseif (is_page(wc_get_page_id('checkout')) && 0 < get_query_var('order-pay')) {

            $order_id = absint(get_query_var('order-pay'));
            $order = wc_get_order($order_id);

            // Test if order needs shipping.
            if (0 < sizeof($order->get_items())) {

                foreach ($order->get_items() as $item) {

                    $_product = $order->get_product_from_item($item);

                    if ($_product && $_product->needs_shipping()) {

                        $needs_shipping = true;
                        break;

                    }

                }

            }

        }

        $needs_shipping = apply_filters('woocommerce_cart_needs_shipping', $needs_shipping);

        // Virtual order, with virtual disabled
        if (!$this->enable_for_virtual && !$needs_shipping) {
            return false;
        }

        // Check methods
        if (!empty($this->enable_for_methods) && $needs_shipping) {

            // Only apply if all packages are being shipped via chosen methods, or order is virtual
            $chosen_shipping_methods_session = WC()->session->get('chosen_shipping_methods');
            $chosen_shipping_methods = isset($chosen_shipping_methods_session) ? array_unique($chosen_shipping_methods_session) : array();
            $check_method = false;

            if (is_object($order)) {

                if ($order->shipping_method) {
                    $check_method = $order->shipping_method;
                }

            } elseif (empty($chosen_shipping_methods) || sizeof($chosen_shipping_methods) > 1) {
                $check_method = false;
            } elseif (sizeof($chosen_shipping_methods) == 1) {
                $check_method = $chosen_shipping_methods[0];
            }

            if (!$check_method) {
                return false;
            }

            $found = false;

            foreach ($this->enable_for_methods as $method_id) {

                if (strpos($check_method, $method_id) === 0) {

                    $found = true;
                    break;

                }

            }

            if (!$found) {
                return false;
            }

        }

        return parent::is_available();

    }

    /**
     * Process the payment and return the result.
     *
     * @since 1.0.0
     * @since 1.0.1 WC 3.0: Update $order->reduce_order_stock() to wc_reduce_stock_levels
     * @access public
     *
     * @param int $order_id WC_Order id
     * @return array
     */
    public function process_payment($order_id) {

        $order = wc_get_order($order_id);

        // Mark as processing or on-hold (payment won't be taken until delivery)
        $order->update_status(apply_filters('igfw_invoice_gateway_process_payment_order_status', $order->has_downloadable_item() ? 'on-hold' : 'processing', $order), __('Payment to be made upon invoice.', 'invoice-gateway-for-woocommerce'));

        // Reduce stock levels
        wc_reduce_stock_levels($order_id);

        // Remove cart
        WC()->cart->empty_cart();

        // Return thankyou redirect
        return array(
            'result' => 'success',
            'redirect' => $this->get_return_url($order),
        );

    }

    /**
     * Output for the order received page.
     *
     * @since 1.0.0
     * @access public
     */
    public function thankyou_page() {

        if ($this->instructions) {
            echo wpautop(wptexturize(esc_attr($this->instructions)));
        }

    }

    /**
     * Add content to the WC emails.
     *
     * @since 1.0.0
     * @since 1.0.1 WC 3.0: Update on how to get payment method. Order properties should not be accessed directly.
     * @access public
     *
     * @access public
     * @param WC_Order $order
     * @param bool     $sent_to_admin
     * @param bool     $plain_text
     */
    public function email_instructions($order, $sent_to_admin, $plain_text = false) {

        if ($this->instructions && !$sent_to_admin && 'igfw_invoice_gateway' === $order->get_payment_method() && $order->get_status() != 'completed') {
            echo wpautop(wptexturize(esc_attr($this->instructions))) . PHP_EOL;
        }

        $invoice_number = get_post_meta($order->get_id(), Plugin_Constants::Invoice_Number, true);

        if ($invoice_number && !$sent_to_admin) {
            if ($plain_text) {
                echo sprintf("\nInvoice Number: %d\n", esc_html($invoice_number));
            } else {
                echo '<span style="color: red; font-weight: 600;"><p>' . sprintf("\nInvoice Number: %d\n", esc_html($invoice_number)) . '</p></span>';
            }

        }

    }

}
